"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const client_1 = require("react-dom/client");
function reactToCustomElement(Component, { strictMode = true, shadowMode, properties = [], attributes = [], }) {
    // https://react.dev/blog/2024/04/25/react-19#support-for-custom-elements
    class CustomElement extends HTMLElement {
        static observedAttributes = [...attributes];
        container;
        root;
        connected = false;
        _componentProps = {};
        _componentAttributes = {};
        _componentEventHandlers = {};
        constructor() {
            super();
            this.container = shadowMode
                ? this.attachShadow({ mode: shadowMode })
                : this;
            this.root = (0, client_1.createRoot)(this.container);
            // React detects if the node has the corresponding properties otherwise
            // props gets set as attributes.
            for (const propertyName of properties) {
                Object.defineProperty(this, propertyName, {
                    enumerable: true,
                    get() {
                        return this._componentProps[propertyName];
                    },
                    set(value) {
                        this._componentProps[propertyName] = value;
                        this.render();
                    },
                });
            }
        }
        connectedCallback() {
            this.connected = true;
            this.render();
        }
        disconnectedCallback() {
            this.connected = false;
            this.root.unmount();
        }
        attributeChangedCallback(attribute, _, value) {
            this._componentAttributes[attribute] = value;
            this.render();
        }
        addEventListener(type, listener, options) {
            for (const propName of properties) {
                // Ignore setting custom event handlers for props named `on***`. Store the callback and pass it directly to the component.
                // https://github.com/facebook/react/blob/9d4fba078812de0363fe9514b943650fa479e8af/packages/react-dom-bindings/src/client/DOMPropertyOperations.js#L189-L231
                if (propName[0] === "o" && propName[1] === "n") {
                    const useCapture = propName.endsWith("Capture");
                    const eventName = propName.slice(2, useCapture ? propName.length - 7 : undefined);
                    if (eventName === type) {
                        // This is an event handler that user passed in.
                        // @ts-expect-error - TODO
                        this._componentEventHandlers[propName] = listener;
                        return;
                    }
                }
            }
            super.addEventListener(type, listener, options);
        }
        render() {
            if (!this.connected) {
                return;
            }
            const Wrapper = strictMode ? react_1.StrictMode : react_1.Fragment;
            this.root.render((0, jsx_runtime_1.jsx)(Wrapper, { children: (0, jsx_runtime_1.jsx)(Component, { ...this._componentAttributes, ...this._componentProps, ...this._componentEventHandlers }) }));
        }
    }
    return CustomElement;
}
exports.default = reactToCustomElement;
